#ifndef __DSP_ODDCAST_H
#define __DSP_ODDCAST_H

#include "Socket.h"
#ifdef HAVE_VORBIS
#include <vorbis/vorbisenc.h>
#endif

#include <stdio.h>
#include <time.h>

#ifdef _DMALLOC_
#include <dmalloc.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
#include "resample.h"
#ifdef __cplusplus
}
#endif

#include "log.h"

#ifdef WIN32
#include <BladeMP3EncDLL.h>
#else
#ifdef HAVE_LAME
#include <lame/lame.h>
#endif
#endif

#ifdef HAVE_FAAC
#undef MPEG2 // hack *cough* hack
typedef signed int  int32_t; 
#include <faac.h>
#endif

#ifdef HAVE_AACP
#include "enc_if.h"
typedef AudioCoder* (*CREATEAUDIO3TYPE)(int , int , int , unsigned int , unsigned int *, char *);
typedef unsigned int (*GETAUDIOTYPES3TYPE)(int, char *);
#endif


#ifdef HAVE_FLAC
#include <OggFLAC/stream_encoder.h>
extern "C" {
#include <FLAC/metadata.h>
}
#endif

#define FormatID 'fmt '   /* chunkID for Format Chunk. NOTE: There is a space at the end of this ID. */
// For skin stuff
#define WINDOW_WIDTH		276
#define WINDOW_HEIGHT		150

#ifndef FALSE
#define FALSE false
#endif
#ifndef TRUE
#define TRUE true
#endif
#ifndef WIN32
#include <sys/ioctl.h>
#else
#include <mmsystem.h>
#endif
// Callbacks
#define	BYTES_PER_SECOND 1

#define FRONT_END_ODDCAST_PLUGIN 1
#define FRONT_END_TRANSCODER 2

typedef struct tagLAMEOptions {
	int		cbrflag;
	int		out_samplerate;
	int		quality;
#ifdef WIN32
	int		mode;
#else
#ifdef HAVE_LAME
	MPEG_mode	mode;
#endif
#endif
	int		brate;
	int		copywrite;
	int		original;
	int		strict_ISO;
	int		disable_reservoir;
	char		VBR_mode[25];
	int		VBR_mean_bitrate_kbps;
	int		VBR_min_bitrate_kbps;
	int		VBR_max_bitrate_kbps;
	int		lowpassfreq;
	int		highpassfreq;
} LAMEOptions;

typedef struct {
	char	RIFF[4];
	long	chunkSize;
	char	WAVE[4];
} RIFFChunk;

typedef struct {
  char		chunkID[4];
  long		chunkSize;

  short          wFormatTag;
  unsigned short wChannels;
  unsigned long  dwSamplesPerSec;
  unsigned long  dwAvgBytesPerSec;
  unsigned short wBlockAlign;
  unsigned short wBitsPerSample;

/* Note: there may be additional fields here, depending upon wFormatTag. */

} FormatChunk;


typedef struct {
	char	chunkID[4];
	long	chunkSize;
	short *	waveformData;
} DataChunk;

struct wavhead
{
	unsigned int  main_chunk;
	unsigned int  length;
	unsigned int  chunk_type;
	unsigned int  sub_chunk;
	unsigned int  sc_len;
	unsigned short  format;
	unsigned short  modus;
	unsigned int  sample_fq;
	unsigned int  byte_p_sec;
	unsigned short  byte_p_spl;
	unsigned short  bit_p_spl;
	unsigned int  data_chunk;
	unsigned int  data_length;
};


static struct wavhead   wav_header;

// Global variables....gotta love em...
typedef struct {
	long		currentSamplerate;
	int		currentBitrate;
	int		currentBitrateMin;
	int		currentBitrateMax;
	int		currentChannels;
	int		gSCSocket;
	int		gSCSocket2;
	int		gSCSocketControl;
	CMySocket	dataChannel;
	CMySocket	controlChannel;
	int		gSCFlag;
	int		gCountdown;
	int		gAutoCountdown;
	int		automaticconnect;
	char		gSourceURL[1024];
	char		gServer[256];
	char		gPort[10];
	char		gPassword[256];
	int		weareconnected;
	char		gIniFile[1024];
	char		gAppName[256];
	char		gCurrentSong[1024];
	int			gSongUpdateCounter;
	char		gMetadataUpdate[10];
	int			gPubServ;
	char		gServIRC[20];
	char		gServICQ[20];
	char		gServAIM[20];
	char		gServURL[1024];
	char		gServDesc[1024];
	char		gServName[1024];
	char		gServGenre[100];
	char		gMountpoint[100];
	char		gFrequency[10];
	char		gChannels[10];
	int			gAutoReconnect;
	int 		gReconnectSec;
	char		gAutoStart[10];
	char		gAutoStartSec[20];
	char		gQuality[5];
#ifndef WIN32
#ifdef HAVE_LAME
	lame_global_flags *gf;
#endif
#endif
	int		gCurrentlyEncoding;
	int		gFLACFlag;
	int		gAACFlag;
	int		gAACPFlag;
	int		gOggFlag;
	char		gIceFlag[10];
	int		gLAMEFlag;
	char		gOggQuality[25];
	int			gLiveRecordingFlag;
	int		gOggBitQualFlag;
	char	gOggBitQual[40];
	char	gEncodeType[25];
	int		gAdvancedRecording;
	int		gNOggchannels;
	char		gModes[4][255];
	int		gShoutcastFlag;
	int		gIcecastFlag;
	int		gIcecast2Flag;
	char		gSaveDirectory[1024];
	char		gLogFile[1024];
	int			gLogLevel;
	int		gSaveDirectoryFlag;
	int		gSaveAsWAV;
	FILE		*gSaveFile;
	LAMEOptions	gLAMEOptions;
	int		gLAMEHighpassFlag;
	int		gLAMELowpassFlag;

	int		oggflag;
	int		serialno;
#ifdef HAVE_VORBIS
	ogg_sync_state  oy_stream;
	ogg_packet header_main_save;
	ogg_packet header_comments_save;
	ogg_packet header_codebooks_save;
#endif
	bool		ice2songChange;
	int			in_header;
	long		 written;
	int		vuShow;

	int		gLAMEpreset;
	char	gLAMEbasicpreset[255];
	char	gLAMEaltpreset[255];
	char    gSongTitle[1024];
	char    gManualSongTitle[1024];
	int		gLockSongTitle;
    int     gNumEncoders;

	res_state	resampler;
	int	initializedResampler;
	void (*sourceURLCallback)(void *, void *);
	void (*destURLCallback)(void *, void *);
	void (*serverStatusCallback)(void *, void *);
	void (*generalStatusCallback)(void *, void *);
	void (*writeBytesCallback)(void *, void *);
	void (*serverTypeCallback)(void *, void *);
	void (*serverNameCallback)(void *, void *);
	void (*streamTypeCallback)(void *, void *);
	void (*bitrateCallback)(void *, void *);
	void (*VUCallback)(int, int);
	long	startTime;
	long	endTime;
	char	sourceDescription[255];

	char	gServerType[25];
#ifdef WIN32
	WAVEFORMATEX waveFormat;
	HWAVEIN      inHandle;
	WAVEHDR                 WAVbuffer1;
	WAVEHDR                 WAVbuffer2;
#else
	int	inHandle; // for advanced recording
#endif
	unsigned long result;
	short int WAVsamplesbuffer1[1152*2];
	short int WAVsamplesbuffer2[1152*2];
	bool  areLiveRecording;
	char	gAdvRecDevice[255];
#ifndef WIN32
	char	gAdvRecServerTitle[255];
#endif
	int		gLiveInSamplerate;
#ifdef WIN32
	// These are for the LAME DLL
        BEINITSTREAM            beInitStream;
        BEENCODECHUNK           beEncodeChunk;
        BEDEINITSTREAM          beDeinitStream;
        BECLOSESTREAM           beCloseStream;
        BEVERSION               beVersion;
        BEWRITEVBRHEADER        beWriteVBRHeader;
        HINSTANCE       hDLL;
        HINSTANCE       hFAACDLL;
        HINSTANCE       hAACPDLL;
        DWORD           dwSamples;
        DWORD           dwMP3Buffer;
        HBE_STREAM      hbeStream;
#endif
		char	gConfigFileName[255];
		char	gOggEncoderText[255];
		int		gForceStop;
		char	gCurrentRecordingName[1024];
		long	lastX;
		long	lastY;
		long	lastDummyX;
		long	lastDummyY;

#ifdef HAVE_VORBIS
		ogg_stream_state os;
		vorbis_dsp_state vd;
		vorbis_block     vb;
		vorbis_info      vi;
#endif

		int	frontEndType;
		int	ReconnectTrigger;
#ifdef HAVE_AACP
		CREATEAUDIO3TYPE	CreateAudio3;
		GETAUDIOTYPES3TYPE	GetAudioTypes3;

		AudioCoder *(*finishAudio3)(char *fn, AudioCoder *c);
		void (*PrepareToFinish)(const char *filename, AudioCoder *coder);
		AudioCoder * aacpEncoder;
#endif
#ifdef HAVE_FAAC
		faacEncHandle aacEncoder;
#endif
		unsigned long samplesInput, maxBytesOutput;
		float   *faacFIFO;
		int     faacFIFOendpos;
		char		gAACQuality[25];
		char		gAACCutoff[25];
        int     encoderNumber;
        bool    forcedDisconnect;
        time_t     forcedDisconnectSecs;
        int    autoconnect;
        char    externalMetadata[255];
        char    externalURL[255];
        char    externalFile[255];
        char    externalInterval[25];
        char    *vorbisComments[30];
        int     numVorbisComments;
        char    outputControl[255];
        char    metadataAppendString[255];
        char    metadataRemoveStringBefore[255];
        char    metadataRemoveStringAfter[255];
        char    metadataWindowClass[255];
        bool    metadataWindowClassInd;
#ifdef HAVE_FLAC
		OggFLAC__StreamEncoder	*flacEncoder;
		FLAC__StreamMetadata *flacMetadata;
		int	flacFailure;
#endif
} oddcastGlobals;



int initializeencoder(oddcastGlobals *g);
void getCurrentSongTitle(oddcastGlobals *g, char *song, char *artist, char *full);
void initializeGlobals(oddcastGlobals *g);
void ReplaceString(char *source, char *dest, char *from, char *to);
void config_read(oddcastGlobals *g);
void config_write(oddcastGlobals *g);
int connectToServer(oddcastGlobals *g);
int disconnectFromServer(oddcastGlobals *g);
int do_encoding(oddcastGlobals *g, short int *samples, int numsamples, int nch);
void URLize(char *input, char *output, int inputlen, int outputlen);
void updateSongTitle(oddcastGlobals *g, int forceURL);
int setCurrentSongTitleURL(oddcastGlobals *g, char *song);
void icecast2SendMetadata(oddcastGlobals *g);
int ogg_encode_dataout(oddcastGlobals *g);
int	trimVariable(char *variable);
int readConfigFile(oddcastGlobals *g,int readOnly = 0);
int writeConfigFile(oddcastGlobals *g);
void    printConfigFileValues();
void ErrorMessage(char *title, char *fmt, ...);
int setCurrentSongTitle(oddcastGlobals *g,char *song);
char*   getSourceURL(oddcastGlobals *g);
void    setSourceURL(oddcastGlobals *g,char *url);
long    getCurrentSamplerate(oddcastGlobals *g);
int     getCurrentBitrate(oddcastGlobals *g);
int     getCurrentChannels(oddcastGlobals *g);
int  ocConvertAudio(oddcastGlobals *g,float *in_samples, float *out_samples, int num_in_samples, int num_out_samples);
int initializeResampler(oddcastGlobals *g,long inSampleRate, long inNCH);
int handle_output(oddcastGlobals *g, float *samples, int nsamples, int nchannels, int in_samplerate);
void setServerStatusCallback(oddcastGlobals *g,void (*pCallback)(void *,void *));
void setGeneralStatusCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setWriteBytesCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setServerTypeCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setServerNameCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setStreamTypeCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setBitrateCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setVUCallback(oddcastGlobals *g, void (*pCallback)(int, int));
void setSourceURLCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setDestURLCallback(oddcastGlobals *g, void (*pCallback)(void *,void *));
void setSourceDescription(oddcastGlobals *g, char *desc);
int  getOggFlag(oddcastGlobals *g);
bool  getLiveRecordingFlag(oddcastGlobals *g);
void setLiveRecordingFlag(oddcastGlobals *g, bool flag);
void setDumpData(oddcastGlobals *g, int dump);
void setConfigFileName(oddcastGlobals *g, char *configFile);
char *getConfigFileName(oddcastGlobals *g);
char*	getServerDesc(oddcastGlobals *g);
int	getReconnectFlag(oddcastGlobals *g);
int getReconnectSecs(oddcastGlobals *g);
int getIsConnected(oddcastGlobals *g);
int resetResampler(oddcastGlobals *g);
void setOggEncoderText(oddcastGlobals *g, char *text);
int getLiveRecordingSetFlag(oddcastGlobals *g);
char *getCurrentRecordingName(oddcastGlobals *g);
void setCurrentRecordingName(oddcastGlobals *g, char *name);
void setForceStop(oddcastGlobals *g, int forceStop);
long	getLastXWindow(oddcastGlobals *g);
long	getLastYWindow(oddcastGlobals *g);
void	setLastXWindow(oddcastGlobals *g, long x);
void	setLastYWindow(oddcastGlobals *g, long y);
long	getLastDummyXWindow(oddcastGlobals *g);
long	getLastDummyYWindow(oddcastGlobals *g);
void	setLastDummyXWindow(oddcastGlobals *g, long x);
void	setLastDummyYWindow(oddcastGlobals *g, long y);
long	getVUShow(oddcastGlobals *g);
void	setVUShow(oddcastGlobals *g, long x);
int		getFrontEndType(oddcastGlobals *g);
void	setFrontEndType(oddcastGlobals *g, int x);
int		getReconnectTrigger(oddcastGlobals *g);
void	setReconnectTrigger(oddcastGlobals *g, int x);
char *getCurrentlyPlaying(oddcastGlobals *g);
long GetConfigVariableLong(char *appName, char *paramName, long defaultvalue, char *desc);
char	*getLockedMetadata(oddcastGlobals *g);
void	setLockedMetadata(oddcastGlobals *g, char *buf);
int		getLockedMetadataFlag(oddcastGlobals *g);
void	setLockedMetadataFlag(oddcastGlobals *g, int flag);
void	setSaveDirectory(oddcastGlobals *g, char *saveDir);
char *getSaveDirectory(oddcastGlobals *g);
char *getgLogFile(oddcastGlobals *g);
void	setgLogFile(oddcastGlobals *g,char *logFile);
int getSaveAsWAV(oddcastGlobals *g);
void setSaveAsWAV(oddcastGlobals *g, int flag);
FILE *getSaveFileP(oddcastGlobals *g);
long getWritten(oddcastGlobals *g);
void setWritten(oddcastGlobals *g, long writ);
int deleteConfigFile(oddcastGlobals *g);
void	setAutoConnect(oddcastGlobals *g, int flag);
void addVorbisComment(oddcastGlobals *g, char *comment);
void freeVorbisComments(oddcastGlobals *g);

#endif
